package GUI;

/*
 OpenCalc is a Graphing Calculator for the web.
 Copyright (C) 2009, 2010 Jason Altekruse

 This file is part of OpenCalc.

 OpenCalc is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 OpenCalc is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with OpenCalc  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JPanel;

import calc.Function;
import calc.Var;
import calc.calc;

public class Graph extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * 
	 */
	private static int X_SIZE;
	private static int Y_SIZE;
	private int LINE_SIZE;
	private double X_MAX;
	private double X_MIN;
	private double Y_MAX;
	private double Y_MIN;
	private double X_STEP;
	private double Y_STEP;
	private double X_PIXEL;
	private double Y_PIXEL;
	private double THETA_STEP;
	private double THETA_MIN;
	private double THETA_MAX;
	private double POL_STEP;
	private double POL_AX_STEP;
	public int mouseX;
	public int mouseY;
	public int mouseRefX;
	public int mouseRefY;
	private boolean refPoint;
	private int NUM_GRAPHS;
	private calc CURRCALC;
	private NewCalc calcObj;
	private JPanel graph;
	// stuff to put in separate graph class
	private Function[] functions;

	public Graph(int xSize, int ySize, NewCalc currCalcObj) {
		calcObj = currCalcObj;
		X_SIZE = xSize;
		Y_SIZE = ySize;
		NUM_GRAPHS = 5;
		LINE_SIZE = 2;
		X_MAX = 10;
		X_MIN = -10;
		Y_MAX = 10;
		Y_MIN = -10;
		X_STEP = 1;
		Y_STEP = 1;
		POL_AX_STEP = 1;
		X_PIXEL = (X_MAX - X_MIN) / X_SIZE;
		Y_PIXEL = (Y_MAX - Y_MIN) / Y_SIZE;
		POL_STEP = Math.PI/360;
		CURRCALC = calcObj.getCalcObj();
		// to be new class for graph
		functions = new Function[NUM_GRAPHS];
		
		for( int i = 0; i < NUM_GRAPHS; i++){
			functions[i] = new Function(CURRCALC);
		}
		
		this.addMouseListener(new MouseListener(){
			private int xStart, yStart;
			
			@Override
			public void mouseClicked(MouseEvent e) {
				// TODO Auto-generated method stub
			}

			@Override
			public void mouseEntered(MouseEvent e) {
				// TODO Auto-generated method stub
				refPoint = true;
				mouseRefX = e.getX();
				mouseRefY= e.getY();
			}

			@Override
			public void mouseExited(MouseEvent e) {
				// TODO Auto-generated method stub
				refPoint = false;
				repaint();
			}

			@Override
			public void mousePressed(MouseEvent e) {
				// TODO Auto-generated method stub
				mouseX = e.getX();
				mouseY = e.getY();
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			
		});
		
		this.addMouseMotionListener(new MouseMotionListener(){
			
			@Override
			public void mouseDragged(MouseEvent e) {
				// TODO Auto-generated method stub
				
				CURRCALC.getVarList().updateVarVal("xMin", (mouseX - e.getX())*X_PIXEL);
				CURRCALC.getVarList().updateVarVal("xMax", (mouseX - e.getX())*X_PIXEL);
				CURRCALC.getVarList().updateVarVal("yMin", (e.getY()- mouseY)*Y_PIXEL);
				CURRCALC.getVarList().updateVarVal("yMax", (e.getY()- mouseY)*Y_PIXEL);
				repaint();
				mouseX = e.getX();
				mouseY = e.getY();
			}

			@Override
			public void mouseMoved(MouseEvent e) {
				// TODO Auto-generated method stub
				mouseRefX = e.getX();
				mouseRefY= e.getY();
				repaint();
			}
			
		});
		
		this.addMouseWheelListener(new MouseWheelListener(){

			@Override
			public void mouseWheelMoved(MouseWheelEvent e) {
				// TODO Auto-generated method stub
				zoom(100 - e.getWheelRotation() * 5);
			}
			
		});
	}
	 /**
	  * 
	  * @param rate
	  */
	public void zoom(double rate){
		double xMin = CURRCALC.getVarList().getVarVal("xMin");
		double xMax = CURRCALC.getVarList().getVarVal("xMax");
		double yMin = CURRCALC.getVarList().getVarVal("yMin");
		double yMax = CURRCALC.getVarList().getVarVal("yMax");
		
		CURRCALC.getVarList().setVarVal("xMin", xMin * 100/(rate));
		CURRCALC.getVarList().setVarVal("xMax", xMax * 100/(rate));
		CURRCALC.getVarList().setVarVal("yMin", yMin * 100/(rate));
		CURRCALC.getVarList().setVarVal("yMax", yMax * 100/(rate));
		repaint();
	}
	
	public Function[] getFunctions(){
		return functions;
	}

	public void setLineSize(int sizeInPixels) {
		LINE_SIZE = sizeInPixels;
		return;
	}

	public void paint(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, X_SIZE, Y_SIZE);
		g.setColor(Color.BLACK);
		X_MIN = CURRCALC.getVarList().getVarVal("xMin");
		X_MAX = CURRCALC.getVarList().getVarVal("xMax");
		Y_MIN = CURRCALC.getVarList().getVarVal("yMin");
		Y_MAX = CURRCALC.getVarList().getVarVal("yMax");
		X_STEP = CURRCALC.getVarList().getVarVal("xStep");
		Y_STEP = CURRCALC.getVarList().getVarVal("yStep");
		THETA_MIN = CURRCALC.getVarList().getVarVal("thetaMin");
		THETA_MAX = CURRCALC.getVarList().getVarVal("thetaMax");
		THETA_STEP = CURRCALC.getVarList().getVarVal("thetaStep");
		//System.out.println("tMin:" + THETA_MIN + " TMAx:" + THETA_MAX + " TStep:" + THETA_STEP);
		X_PIXEL = (X_MAX - X_MIN) / X_SIZE;
		Y_PIXEL = (Y_MAX - Y_MIN) / Y_SIZE;

		drawAxis(g);
		//drawPolarAxis(g);
		
		Function f = null;
		for (int i = 0; i < NUM_GRAPHS; i++){
			f = functions[i];
			if (f != null){
				if (f.isGraphing()){
					if(f.getGraphType() == 1){
						graphCart(f, g);
					}
					else if(f.getGraphType() == 2){
						graphPolar(f, g);
					}
					else if(f.getGraphType() == 4){
						graphCart(f, g);
					}
				}
					
			}
		}
		if(refPoint)
			drawMousePlaceMent(g);

	}

	private void drawMousePlaceMent(Graphics g){
		float currX = (float) ((mouseRefX * X_PIXEL) + X_MIN);
		float currY = (float) (Y_MAX - (mouseRefY * Y_PIXEL));
		String ptText = "(" + currX + ", " + currY + ")";
		int width = g.getFontMetrics().stringWidth(ptText);
		g.setColor(Color.white);
		g.fillRect( X_SIZE - width - 5, Y_SIZE - 20, width, 12);
		g.setColor(Color.black);
		g.drawString(ptText, X_SIZE - width - 5, Y_SIZE - 10);
	}
	
	private void ptOn(double a, double b, Graphics g) {
		if (a <= X_MAX && a >= X_MIN && b <= Y_MAX && b >= Y_MIN) {
			g.fillRect(roundDouble((a - X_MIN) / X_PIXEL) - LINE_SIZE,
					(Y_SIZE - LINE_SIZE) - roundDouble((b - Y_MIN) / Y_PIXEL),
					LINE_SIZE, LINE_SIZE);
			// g.fillOval(roundDouble((a - X_MIN)/X_PIXEL), Y_SIZE -
			// roundDouble((b - Y_MIN)/Y_PIXEL),
			// 2*LINE_SIZE, 2*LINE_SIZE);
			// System.out.println("(" + a + "," + b + ")");
			// System.out.println("(" + roundDouble(a/X_PIXEL) + "," +
			// roundDouble(b/Y_PIXEL) + ")" + "\n");
		}
	}
	
	private void polPtOn(double r, double theta, Graphics g){
		ptOn(r*Math.cos(theta), r*Math.sin(theta), g);
	}

	private void drawPolarAxis(Graphics g){
		
		LINE_SIZE = 1;
		int numCircles = 0; 
		if (X_MAX > Y_MAX || X_MAX > Math.abs(Y_MIN))
			numCircles = (int) Math.abs(X_MAX - X_MIN);
		else if (Y_MAX > X_MAX || Y_MAX > Math.abs(X_MIN)){
			numCircles = (int) Math.abs(Y_MAX - Y_MIN);
		}
		else
			numCircles = (int) X_MAX;
		int i = roundDouble(Y_MIN);
		for ( ; i <= numCircles; i++){
			double currT = 0;
			double lastX = i * POL_AX_STEP;
			double lastY = 0, currX, currY;
			for(int j = 1; j < 240; j++){
				currT += POL_STEP * 2;
				currX = i * Math.cos(currT);
				currY = i * Math.sin(currT);
				drawLineSeg(lastX, lastY, currX, currY, Color.gray, g);
				lastX = currX;
				lastY = currY;
			}
		}
	}
	
	private void drawAxis(Graphics g) {

		// finds the fist factor of the X_STEP on the screen
		// used to draw the first dash mark on the x-axis
		double tempX = (int) (X_MIN / X_STEP);
		tempX *= X_STEP;

		if (Y_MIN <= 0 && Y_MAX >= 0) {
			while (tempX < X_MAX) {
				setLineSize(1);
				drawLineSeg(tempX, Y_MIN, tempX, Y_MAX, Color.GRAY, g);
				setLineSize(2);
				drawLineSeg(tempX, 2 * LINE_SIZE * Y_PIXEL, tempX, -1
						* LINE_SIZE * Y_PIXEL, Color.BLACK, g);
				tempX += X_STEP;
			}
		} else {
			if( Y_MIN >= 0){
				while (tempX < X_MAX) {
					setLineSize(1);
					drawLineSeg(tempX, Y_MIN, tempX, Y_MAX, Color.GRAY, g);
					setLineSize(2);
					g.setColor(Color.BLACK);
					ptOn(tempX, Y_MIN, g);
					tempX += X_STEP;
				}
			}
			if( Y_MAX <= 0){
				while (tempX < X_MAX) {
					setLineSize(1);
					drawLineSeg(tempX, Y_MIN, tempX, Y_MAX, Color.GRAY, g);
					setLineSize(2);
					g.setColor(Color.BLACK);
					ptOn(tempX, Y_MAX - 1 * LINE_SIZE * Y_PIXEL, g);
					tempX += X_STEP;
				}
			}
		}

		// finds the fist factor of the Y_STEP on the screen
		// used to draw the first dash mark on the y-axis
		double tempY = (int) (Y_MIN / Y_STEP);
		tempY *= Y_STEP;

		if (X_MIN <= 0 && X_MAX >= 0) {
			while (tempY < Y_MAX) {
				setLineSize(1);
				drawLineSeg(X_MIN, tempY, X_MAX, tempY, Color.GRAY, g);
				setLineSize(2);
				drawLineSeg(2 * LINE_SIZE * X_PIXEL, tempY, -1 * LINE_SIZE
						* X_PIXEL, tempY, Color.BLACK, g);
				tempY += Y_STEP;
			}
		} else {
			if(X_MIN >= 0){
				while (tempY < Y_MAX) {
					setLineSize(1);
					drawLineSeg(X_MIN, tempY, X_MAX, tempY, Color.GRAY, g);
					setLineSize(2);
					g.setColor(Color.BLACK);
					ptOn(X_MIN + 5 * X_PIXEL, tempY, g);
					tempY += Y_STEP;
				}
			}
			else if(X_MAX <= 0){
				while (tempY < Y_MAX) {
					setLineSize(1);
					drawLineSeg(X_MIN, tempY, X_MAX, tempY, Color.GRAY, g);
					setLineSize(2);
					g.setColor(Color.BLACK);
					ptOn(X_MAX - 3 * X_PIXEL, tempY, g);
					tempY += Y_STEP;
					}
			}
		}

		if (X_MIN <= 0 && X_MAX >= 0)
			drawLineSeg(0, Y_MIN, 0, Y_MAX, Color.BLACK, g);

		if (Y_MIN <= 0 && Y_MAX >= 0)
			drawLineSeg(X_MIN, 0, X_MAX, 0, Color.BLACK, g);
	}

	private void drawLineSeg(double x1, double y1, double x2, double y2,
			Color color, Graphics g) {
		// System.out.println(x1 + "  " + y1 + "  " + x2 + "  " + y2);
		g.setColor(color);
		double smallX = 0, bigX = 0, smallY = 0, bigY = 0;

		if (x1 == x2) {

			if (y1 > y2) {
				bigY = y1;
				smallY = y2;
			} else {
				bigY = y2;
				smallY = y1;
			}

			while (smallY <= bigY - LINE_SIZE * Y_PIXEL) {
				ptOn(x1, smallY, g);
				smallY += Y_PIXEL;
			}
		}

		else if (y1 == y2) {

			if (x1 > x2) {
				bigX = x1;
				smallX = x2;
			} else {
				bigX = x2;
				smallX = x1;
			}

			while (smallX <= bigX - LINE_SIZE * X_PIXEL) {
				ptOn(smallX, y1, g);
				smallX += X_PIXEL;
			}
		}
		else {
			double slope = (y2 - y1) / (x2 - x1);
			double b = y1-(x1*slope);
			
			// System.out.println("slope: " + slope);
				if (Math.abs(slope) > 1) {
					if (y1 > y2) {
						bigX = x1;
						bigY = y1;
						smallX = x2;
						smallY = y2;
					}
					else {
						bigX = x2;
						bigY = y2;
						smallX = x1;
						smallY = y1;
					}
					if(smallY < Y_MIN)
						smallY = Y_MIN;
					smallX = (smallY - b)/slope;
					while (smallY <= Y_MAX && smallY <= bigY) {
						ptOn(smallX, smallY, g);
						smallX += Y_PIXEL / slope;
						smallY += Y_PIXEL;
					}
				}
				else { // absolute value of slope is less than 1
					if (x1 > x2) {
						bigX = x1;
						bigY = y1;
						smallX = x2;
						smallY = y2;
					}
					else {
						bigX = x2;
						bigY = y2;
						smallX = x1;
						smallY = y1;
					}
					if(smallX < X_MIN)
						smallX = X_MIN;
					smallY = smallX * slope  + b;
					while (smallX <= X_MAX && smallX <= bigX) {
						ptOn(smallX, smallY, g);
						smallX += X_PIXEL;
						smallY += X_PIXEL * slope;
					}
				}
		}
	}

	public int roundDouble(double a) {
		if (a % 1 >= .5)
			return (int) a + 1;
		else
			return (int) a;
	}

	public void graphCart(Function f, Graphics g) {
		String eqtn = f.getFuncEqtn();
		Var ind = f.getIndependentVar();
		Var dep = f.getDependentVar();
		Color color = f.getColor();
		boolean tracing = f.isTracingPt();
		double tracePt = f.getTraceVal();
		boolean isConnected = f.isConneted();
		boolean takeIntegral = f.isTakingIntegral();
		double a = f.getStartIntegral();
		double b = f.getEndIntegral();
		
		double lastX, lastY, currX, currY;
		g.setColor(color);
		
		ind.setValue(X_MIN);
		CURRCALC.parse(eqtn);
		CURRCALC.solve();
		lastX = ind.getValue();
		lastY = dep.getValue();
		for (int i = 1; i < X_SIZE; i += 2) {
			ind.updateValue(2 * X_PIXEL);
			CURRCALC.solve();
			currX = ind.getValue();
			currY = dep.getValue();

			ptOn(currX, currY, g);
			if (isConnected){
				drawLineSeg(lastX, lastY, currX, currY, color, g);
			}

			if (takeIntegral) {
				if (currX >= a && currX <= b) {
					if (currY < Y_MAX && currY > Y_MAX)
						drawLineSeg(currX, 0, currX, Y_MAX, color, g);
					else if (currY < Y_MAX && currY > Y_MIN)
						drawLineSeg(currX, 0, currX, currY, color, g);
					else if (currY <= Y_MIN)
						drawLineSeg(currX, 0, currX, Y_MIN, color, g);
					else if (currY >= Y_MAX)
						drawLineSeg(currX, 0, currX, Y_MAX, color, g);
					else
						;// do nothing
				}
			}
			lastX = ind.getValue();
			lastY = dep.getValue();
		}
		if (tracing) {
			g.setColor(Color.black);
			CURRCALC.parse(eqtn);
			ind.setValue(tracePt);
			drawTracer(tracePt, CURRCALC.solve(), g);
		}
	}
	
	public void graphPolar(Function f, Graphics g) {
		String eqtn = f.getFuncEqtn();
		Var ind = f.getIndependentVar();
		Var dep = f.getDependentVar();
		Color color = f.getColor();
		boolean tracing = f.isTracingPt();
		double tracePt = f.getTraceVal();
		boolean isConnected = f.isConneted();
		boolean takeIntegral = f.isTakingIntegral();
		double a = f.getStartIntegral();
		double b = f.getEndIntegral();
		int angleUnits = CURRCALC.getAngleUnits();
		
		double currR, currT, lastX, lastY, currX, currY;
		g.setColor(color);
		
		
		ind.setValue(THETA_MIN);
		CURRCALC.parse(eqtn);
		CURRCALC.solve();
		currR = dep.getValue();
		currT = ind.getValue();
		
		lastX = currR * Math.cos(currT);
		lastY = currR * Math.sin(currT);
		int numCalcs = (int)((THETA_MAX-THETA_MIN)/THETA_STEP);
		for (int i = 1; i <= numCalcs; i++) {
			ind.updateValue(THETA_STEP);
			CURRCALC.solve();
			currR = dep.getValue();
			currT = ind.getValue();
			
			if(angleUnits == 2)
				currT *= (Math.PI/180);
			if(angleUnits == 3)
				currT *= (Math.PI/200);
			currX = currR * Math.cos(currT);
			currY = currR * Math.sin(currT);
			//polPtOn(currT, currR, g);
			drawLineSeg(lastX, lastY, currX, currY, color, g);
			lastX = currX;
			lastY = currY;
		}
	}

	public void drawTracer(double x, double y, Graphics g) {
		ptOn(x, y, g);
		ptOn(x + LINE_SIZE * X_PIXEL, y + LINE_SIZE * Y_PIXEL, g);
		ptOn(x + LINE_SIZE * X_PIXEL, y - LINE_SIZE * Y_PIXEL, g);
		ptOn(x - LINE_SIZE * X_PIXEL, y + LINE_SIZE * Y_PIXEL, g);
		ptOn(x - LINE_SIZE * X_PIXEL, y - LINE_SIZE * Y_PIXEL, g);
	}
	
	public calc getGraphCalc(){
		return CURRCALC;
	}
}
